==> CRF%2B%2B-0.42.tar.gz: CRF++-0.42/thread.h <==
/*
  CRF++ -- Yet Another CRF toolkit
 
  $Id: thread.h,v 1.1 2006/03/28 18:15:59 taku Exp $;

  Copyright (C) 2005 Taku Kudo <taku@chasen.org>

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _CRFPP_THREAD_H
#define _CRFPP_THREAD_H

#ifdef HAVE_PTHREAD_H      
#include <pthread.h>
#else
#ifdef _WIN32
#include <windows.h>
#include <process.h>
#endif
#endif

#if defined HAVE_PTHREAD_H
#define CRFPP_USE_THREAD 1
#endif

#if (defined  (_WIN32) && ! defined (__CYGWIN__))
#define CRFPP_USE_THREAD 1
#endif

#define BEGINTHREAD(src, stack, func, arg, flag, id) \
     (HANDLE)_beginthreadex((void *)(src), (unsigned)(stack), \
		       (unsigned (_stdcall *)(void *))(func), (void *)(arg), \
		       (unsigned)(flag), (unsigned *)(id))

namespace CRFPP {

  class thread
  {
  private:
#ifdef HAVE_PTHREAD_H   
    pthread_t hnd;
#else
#ifdef _WIN32   
    HANDLE  hnd;
#endif
#endif
   
  public:
    static void* wrapper(void *ptr) {
      thread *p = static_cast<thread *>(ptr);
      p->run();
      return 0;
    }

    virtual void run () {}

    void start() {
#ifdef HAVE_PTHREAD_H      
      pthread_create(&hnd, 0, &thread::wrapper, 
		     static_cast<void *>(this));
      
#else
#ifdef _WIN32
      DWORD id;
      hnd = BEGINTHREAD(0, 0, &thread::wrapper, this, 0, &id);
#endif
#endif
    }
   
    void join() {
#ifdef HAVE_PTHREAD_H            
      pthread_join(hnd, 0);
#else
#ifdef _WIN32
      WaitForSingleObject(hnd, INFINITE);
      CloseHandle(hnd);
#endif
#endif      
    }

    virtual ~thread() {};
  };
}
#endif

==> CRF%2B%2B-0.43.tar.gz: CRF++-0.43/thread.h <==
/*
  CRF++ -- Yet Another CRF toolkit

  $Id: thread.h,v 1.3 2006/08/07 02:35:35 taku Exp $;

  Copyright(C) 2005 Taku Kudo <taku@chasen.org>

  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or(at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _CRFPP_THREAD_PTR_H
#define _CRFPP_THREAD_PTR_H

#ifdef HAVE_PTHREAD_H
#include <pthread.h>
#else
#ifdef _WIN32
#include <windows.h>
#include <process.h>
#endif
#endif

#if defined HAVE_PTHREAD_H
#define CRFPP_USE_THREAD 1
#endif

#if(defined(_WIN32) && ! defined (__CYGWIN__))
#define CRFPP_USE_THREAD 1
#endif

#define BEGINTHREAD(src, stack, func, arg, flag, id) \
     (HANDLE)_beginthreadex((void *)(src),(unsigned)(stack), \
                       (unsigned(_stdcall *)(void *))(func),(void *)(arg), \
                       (unsigned)(flag),(unsigned *)(id))

namespace CRFPP {

  class thread {
  private:
#ifdef HAVE_PTHREAD_H
    pthread_t hnd_;
#else
#ifdef _WIN32
    HANDLE  hnd_;
#endif
#endif

  public:
    static void* wrapper(void *ptr) {
      thread *p = static_cast<thread *>(ptr);
      p->run();
      return 0;
    }

    virtual void run() {}

    void start() {
#ifdef HAVE_PTHREAD_H
      pthread_create(&hnd_, 0, &thread::wrapper,
                     static_cast<void *>(this));

#else
#ifdef _WIN32
      DWORD id;
      hnd_ = BEGINTHREAD(0, 0, &thread::wrapper, this, 0, &id);
#endif
#endif
    }

    void join() {
#ifdef HAVE_PTHREAD_H
      pthread_join(hnd_, 0);
#else
#ifdef _WIN32
      WaitForSingleObject(hnd_, INFINITE);
      CloseHandle(hnd_);
#endif
#endif
    }

    virtual ~thread() {};
  };
}


#endif
